#version 130
precision highp float;

// iOrange - uniforms
uniform mat4x4 modelViewProjection;
uniform float bones[126];

// iOrange - incoming params
in vec4 vPos;
in vec4 vIndexes;
in vec4 vWeights;
in vec2 vTexCoord;

// iOrange - outgoing params
out vec2 tc;

vec3 VertexTransform(vec3 p, int index)
{
	int i = index * 7;

	// restore rotation component (quaternion)
	float x = bones[i];
	float y = bones[i+1];
	float z = bones[i+2];
	float w = bones[i+3];
	// restore offset component (vec3)
	float tx = bones[i+4];
	float ty = bones[i+5];
	float tz = bones[i+6];


	// original code from DooM 3 SDK
	float xxzz = x*x - z*z;
	float wwyy = w*w - y*y;
	float xw2 = x*w*2.0;
	float xy2 = x*y*2.0;
	float xz2 = x*z*2.0;
	float yw2 = y*w*2.0;
	float yz2 = y*z*2.0;
	float zw2 = z*w*2.0;
	vec3 ret = vec3((xxzz + wwyy)*p.x		+ (xy2 + zw2)*p.y		+ (xz2 - yw2)*p.z,
					(xy2 - zw2)*p.x			+ (y*y+w*w-x*x-z*z)*p.y	+ (yz2 + xw2)*p.z,
					(xz2 + yw2)*p.x			+ (yz2 - xw2)*p.y		+ (wwyy - xxzz)*p.z);

	return ret + vec3(tx, ty, tz);
}

void main()
{
	tc = vTexCoord;
	vec4 p = vec4(0.0, 0.0, 0.0, 1.0);
	p.xyz = VertexTransform(vPos.xyz, int(vIndexes.x)) * vWeights.x +
			VertexTransform(vPos.xyz, int(vIndexes.y)) * vWeights.y +
			VertexTransform(vPos.xyz, int(vIndexes.z)) * vWeights.z +
			VertexTransform(vPos.xyz, int(vIndexes.w)) * vWeights.w;
	gl_Position = modelViewProjection * p;
}
